==> pawm-2.2.7.tar.bz2: pawm-2.2.7/src/button.c <==
/*
    Filename: button.c
    Synopsis: Pretty Button widget.

                          Copyright (C) 2000-2006
            Ral Nez de Arenas Coronado <raul@pleyades.net>
				David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to:

        Free Software Foundation, Inc.
        59 Temple Place, Suite 330
        Boston, MA 02111-1307 USA

*/

#include "x.h"
#include "button.h"
#include "mobs.h"


static unsigned short int reference_count=0;

#define BUTTON_EVMASK (ButtonPressMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask| ExposureMask|ResizeRedirectMask)




/* Data Structures *********************************************************/

struct sButton {
	Window window;       /* Button's window */
	unsigned int w;      /* Button's width */
	unsigned int h;      /* Button's height */
	Bool active;         /* Button is active if true */
	Bool pressd;         /* Button is pressed if true */
	callback action;     /* What to do when depressing */
	void *param;         /* Parameter to the function */
	Pixmap n;            /* Face for NORMAL state */
	Pixmap a;            /* Face for ACTIVE state */
	Pixmap p;            /* Face for PRESSED state */
};

#define ButtonBytes sizeof(struct sButton)




/* Function prototypes *****************************************************/
/* Handlers */
static void button_handle (XEvent *, void *);
static void button_ButtonPress (XButtonPressedEvent *, Button);
static void button_ButtonRelease (XButtonReleasedEvent *, Button);
static void button_EnterNotify (Button);
static void button_LeaveNotify (Button);
static void button_Expose (XExposeEvent *, Button);

/* Helpers */
static void draw_all (Button);
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/
static void button_handle (XEvent *xevent, void *widget)
{
	Button button=NULL;

	ASSERT(xevent);
	ASSERT(widget);
		
	button=(Button)widget;

	switch(xevent->type) {
		case ButtonPress:
			GOT(ButtonPress, Button);
			button_ButtonPress(&(xevent->xbutton), button);
			break;
		case ButtonRelease:
			GOT(ButtonRelease, Button);
			button_ButtonRelease(&(xevent->xbutton), button);
			break;
		case EnterNotify:
			GOT(EnterNotify, Button);
			button_EnterNotify(button);
			break;
		case LeaveNotify:
			GOT(LeaveNotify, Button);
			button_LeaveNotify(button);
			break;
		case Expose:
			GOT(Expose, Button);
			button_Expose(&(xevent->xexpose), button);
			break;
	}
}



static void button_ButtonPress (XButtonPressedEvent *event, Button button)
{
	ASSERT(event);
	ASSERT(button);    

	if (event->button!=Button1) return;

	button->active=True;
	button->pressd=True;
	draw_all(button);
}



static void button_ButtonRelease (XButtonReleasedEvent *event, Button button)
{
	ASSERT(event);
	ASSERT(button);

	if (event->button!=Button1) return;

	if (button->pressd==False) return;
	button->pressd=False;
	draw_all(button);

	/* Do action, if any */
	if ((button->active==True) && (button->action))
		button->action(button->param);
}



/* Hilite the button */
static void button_EnterNotify (Button button)
{
	ASSERT(button);

	/* Put the 'ACTIVE' state Pixmap */
	button->active=True;
	draw_all(button);
}



static void button_LeaveNotify (Button button)
{
    ASSERT(button);

    button->active=False;
    draw_all(button);
}




static void button_Expose (XExposeEvent *event, Button button)
{
	ASSERT(event);
	ASSERT(button);

	if (event->count) return;  /* More exposes to come */

	draw_all(button);
}
/****************************************************************************/





/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/
static void draw_all (Button button)
{
	ASSERT(button);

	if (button->pressd) 
		XCopyArea(display, button->p, button->window, xgc, 0, 0, button->w, button->h, 0, 0);
	else if (button->active) 
		XCopyArea(display, button->a, button->window, xgc, 0, 0, button->w, button->h, 0, 0);
	else
		XCopyArea(display, button->n, button->window, xgc, 0, 0, button->w, button->h, 0, 0);

}

/****************************************************************************/






/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
void button_init (void)
{
    if (reference_count++) return;

}



void button_done (void)
{
    if (--reference_count) return;
    
}



/* Creates a new button with geometry 'width*height+x+y', which should
 * be destroyed with a call to 'button_destroy()' when no longer needed.
 * The newly created button is returned on success, or NULL on error.
 * Please note that the button is not mapped into the screen.

 * The button size (its geometry) is specified by the 'width' and 'height'
 * parameters. There is no point in specifying the range, since the function
 * will surely fail if the window is so small or so big....

 * The Pixmaps are the faces of the buttons. All three must be provided,
 * although they can be identical, no matter about that. */
Button button_create (Window parent, int x, int y, unsigned int width, unsigned int height,
                      Pixmap nface, Pixmap hface, Pixmap pface, callback action, void *param) 
{
	XSizeHints *sizehints;
	XSetWindowAttributes attributes;
	Button button=NULL;

	ASSERT(parent!=None);
	ASSERT(nface!=None); 
	ASSERT(hface!=None);
	ASSERT(pface!=None);

	/** Button initialization **/

	/* Create the 'Button' structure */
	button=malloc(ButtonBytes);
	if (!button) die("Not enough memory");

	/* Fill the fields with default values */        
	button->w=width;
	button->h=height;
	button->action=action;
	button->param=param;
	button->n=nface;
	button->a=hface;
	button->p=pface;


	/** Window creation and basic setup **/

	XSetWindowAttributes wattr;
	unsigned long wmask;

	wattr.border_pixel=0;
	wattr.colormap = XCreateColormap (display, ROOT, DEFAULTVISUAL, AllocNone);
	wattr.event_mask= ExposureMask|StructureNotifyMask;
	wmask = CWEventMask | CWBorderPixel | CWColormap;
	
	button->window=XCreateWindow(display, parent, x, y, button->w, button->h,
								  	   0, DEPTH, InputOutput, DEFAULTVISUAL, wmask, &wattr);
	//if (visuals.argb_visual)	
	//	x_alpha_gc_init(button->window);	

	if (button->window==None) BUG();
		
    XSetGraphicsExposures(display, xgc, False);

	/* Now put the button a name */
	XStoreName(display, button->window, "button");

	/* Make the window not resizeable and mark as transient */
	sizehints=XAllocSizeHints();
	sizehints->min_width =button->w;
	sizehints->max_width =button->w;
	sizehints->min_height=button->h;
	sizehints->max_height=button->h;
	sizehints->base_width =button->w;
	sizehints->base_height=button->h;
	sizehints->min_aspect.x=button->w;
	sizehints->min_aspect.y=button->h;
	sizehints->max_aspect.x=button->w;
	sizehints->max_aspect.y=button->h;

	/* Set the hints */
	sizehints->flags=PPosition|PMinSize|PMaxSize|PAspect|PBaseSize;
	XSetWMNormalHints(display, button->window, sizehints);
	XSetTransientForHint(display, button->window, button->window);
	attributes.override_redirect=True;
	XChangeWindowAttributes(display, button->window, CWOverrideRedirect, &attributes);


	/** Final setup **/
	button->active=False;
	button->pressd=False;

	/* Select the event mask for the button */
	XSelectInput(display, button->window, BUTTON_EVMASK);
	x_addsloot(button->window, button_handle, button);
		
	return button;
}




/*
    This function destroys a Button widget. Take into account that the
widget will be invalid and unusable after the call. Caveat emptor.
*/
void button_destroy (Button button)
{
	ASSERT(button);

	XUndefineCursor(display, button->window);
	XDestroyWindow(display, button->window);

	memset(button, '\0', ButtonBytes);
	free(button);

	button=NULL;
}




void button_map (Button button)
{
	ASSERT(button);

	XMapWindow(display, button->window);
}




void button_unmap (Button button)
{
	ASSERT(button);
		
	XUnmapWindow(display, button->window);
}




void button_move (Button button, int x, int y)
{
	ASSERT(button);
		
	XMoveWindow(display, button->window, x, y);
}



void button_reparent (Button button, Window parent, int x, int y)
{
	ASSERT(button);
	if (parent==None) return;

	/* Make sure the button state is correct */
	button->pressd=False;

	XReparentWindow(display, button->window, parent, x, y);
}




void button_action (Button button, callback action, void *param)
{
	ASSERT(button);

	button->action=action;
	button->param=param;
}

Window button_getwindow (Button button) 
{
	ASSERT(button);

	return button->window;
}

==> pawm-2.3.0.tar.bz2: pawm-2.3.0/src/button.c <==
/*
    Filename: button.c
    Synopsis: Pretty Button widget.

                          Copyright (C) 2000-2006
            Ral Nez de Arenas Coronado <raul@pleyades.net>
				David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License 
	version 2 as published by the Free Software Foundation

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to:

        Free Software Foundation, Inc.
        59 Temple Place, Suite 330
        Boston, MA 02111-1307 USA

*/

#include "x.h"
#include "button.h"


static unsigned short int reference_count=0;

#define BUTTON_EVMASK (ButtonPressMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask| ExposureMask|ResizeRedirectMask)




/* Data Structures *********************************************************/

struct sButton {
	Window window;       /* Button's window */
	unsigned int w;      /* Button's width */
	unsigned int h;      /* Button's height */
	Bool active;         /* Button is active if true */
	Bool pressd;         /* Button is pressed if true */
	callback action;     /* What to do when depressing */
	void *param;         /* Parameter to the function */
	Pixmap n;            /* Face for NORMAL state */
	Pixmap a;            /* Face for ACTIVE state */
	Pixmap p;            /* Face for PRESSED state */
};

#define ButtonBytes sizeof(struct sButton)




/* Function prototypes *****************************************************/
/* Handlers */
static void button_handle (XEvent *, void *);
static void button_ButtonPress (XButtonPressedEvent *, Button);
static void button_ButtonRelease (XButtonReleasedEvent *, Button);
static void button_EnterNotify (Button);
static void button_LeaveNotify (Button);
static void button_Expose (XExposeEvent *, Button);

/* Helpers */
static void draw_all (Button);
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/
static void button_handle (XEvent *xevent, void *widget)
{
	Button button=NULL;

	assert(xevent);
	assert(widget);
		
	button=(Button)widget;

	switch(xevent->type) {
		case ButtonPress:
			GOT(ButtonPress, Button);
			button_ButtonPress(&(xevent->xbutton), button);
			break;
		case ButtonRelease:
			GOT(ButtonRelease, Button);
			button_ButtonRelease(&(xevent->xbutton), button);
			break;
		case EnterNotify:
			GOT(EnterNotify, Button);
			button_EnterNotify(button);
			break;
		case LeaveNotify:
			GOT(LeaveNotify, Button);
			button_LeaveNotify(button);
			break;
		case Expose:
			GOT(Expose, Button);
			button_Expose(&(xevent->xexpose), button);
			break;
	}
}



static void button_ButtonPress (XButtonPressedEvent *event, Button button)
{
	assert(event);
	assert(button);    

	if (event->button!=Button1) return;

	button->active=True;
	button->pressd=True;
	draw_all(button);
}



static void button_ButtonRelease (XButtonReleasedEvent *event, Button button)
{
	assert(event);
	assert(button);

	if (event->button!=Button1) return;

	if (button->pressd==False) return;
	button->pressd=False;
	draw_all(button);

	/* Do action, if any */
	if ((button->active==True) && (button->action))
		button->action(button->param);
}



/* Hilite the button */
static void button_EnterNotify (Button button)
{
	assert(button);

	/* Put the 'ACTIVE' state Pixmap */
	button->active=True;
	draw_all(button);
}



static void button_LeaveNotify (Button button)
{
    assert(button);

    button->active=False;
    draw_all(button);
}




static void button_Expose (XExposeEvent *event, Button button)
{
	assert(event);
	assert(button);

	if (event->count) return;  /* More exposes to come */

	draw_all(button);
}
/****************************************************************************/





/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/
static void draw_all (Button button)
{
	assert(button);

	if (button->pressd) 
		XCopyArea(xclass.display, button->p, button->window, xclass.gc, 0, 0, button->w, button->h, 0, 0);
	else if (button->active) 
		XCopyArea(xclass.display, button->a, button->window, xclass.gc, 0, 0, button->w, button->h, 0, 0);
	else
		XCopyArea(xclass.display, button->n, button->window, xclass.gc, 0, 0, button->w, button->h, 0, 0);

}

/****************************************************************************/






/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
void button_init (void)
{
    if (reference_count++) return;

}



void button_done (void)
{
    if (--reference_count) return;
    
}



/* Creates a new button with geometry 'width*height+x+y', which should
 * be destroyed with a call to 'button_destroy()' when no longer needed.
 * The newly created button is returned on success, or NULL on error.
 * Please note that the button is not mapped into the screen.

 * The button size (its geometry) is specified by the 'width' and 'height'
 * parameters. There is no point in specifying the range, since the function
 * will surely fail if the window is so small or so big....

 * The Pixmaps are the faces of the buttons. All three must be provided,
 * although they can be identical, no matter about that. */
Button button_create (Window parent, int x, int y, unsigned int width, unsigned int height,
                      Pixmap nface, Pixmap hface, Pixmap pface, callback action, void *param) 
{
	XSizeHints *sizehints;
	XSetWindowAttributes attributes;
	Button button=NULL;

	assert(parent!=None);
	assert(nface!=None); 
	assert(hface!=None);
	assert(pface!=None);

	/** Button initialization **/

	/* Create the 'Button' structure */
	button=malloc(ButtonBytes);
	if (!button) die("Not enough memory");

	/* Fill the fields with default values */        
	button->w=width;
	button->h=height;
	button->action=action;
	button->param=param;
	button->n=nface;
	button->a=hface;
	button->p=pface;


	/** Window creation and basic setup **/

	XSetWindowAttributes wattr;
	unsigned long wmask;

	wattr.border_pixel=0;
	wattr.colormap = XCreateColormap (xclass.display, ROOT, DEFAULTVISUAL, AllocNone);
	wattr.event_mask= ExposureMask|StructureNotifyMask;
	wmask = CWEventMask | CWBorderPixel | CWColormap;
	
	button->window=XCreateWindow(xclass.display, parent, x, y, button->w, button->h,
								  	   0, DEPTH, InputOutput, DEFAULTVISUAL, wmask, &wattr);
	//if (visuals.argb_visual)	
	//	x_alpha_gc_init(button->window);	

	if (button->window==None) BUG();
		
    XSetGraphicsExposures(xclass.display, xclass.gc, False);

	/* Now put the button a name */
	XStoreName(xclass.display, button->window, "button");

	/* Make the window not resizeable and mark as transient */
	sizehints=XAllocSizeHints();
	sizehints->min_width =button->w;
	sizehints->max_width =button->w;
	sizehints->min_height=button->h;
	sizehints->max_height=button->h;
	sizehints->base_width =button->w;
	sizehints->base_height=button->h;
	sizehints->min_aspect.x=button->w;
	sizehints->min_aspect.y=button->h;
	sizehints->max_aspect.x=button->w;
	sizehints->max_aspect.y=button->h;

	/* Set the hints */
	sizehints->flags=PPosition|PMinSize|PMaxSize|PAspect|PBaseSize;
	XSetWMNormalHints(xclass.display, button->window, sizehints);
	XSetTransientForHint(xclass.display, button->window, button->window);
	attributes.override_redirect=True;
	XChangeWindowAttributes(xclass.display, button->window, CWOverrideRedirect, &attributes);


	/** Final setup **/
	button->active=False;
	button->pressd=False;

	/* Select the event mask for the button */
	XSelectInput(xclass.display, button->window, BUTTON_EVMASK);
	x_addsloot(button->window, button_handle, button);
		
	return button;
}




/*
    This function destroys a Button widget. Take into account that the
widget will be invalid and unusable after the call. Caveat emptor.
*/
void button_destroy (Button button)
{
	assert(button);

	XUndefineCursor(xclass.display, button->window);
	XDestroyWindow(xclass.display, button->window);

	memset(button, '\0', ButtonBytes);
	free(button);

	button=NULL;
}




void button_map (Button button)
{
	assert(button);

	XMapWindow(xclass.display, button->window);
}




void button_unmap (Button button)
{
	assert(button);
		
	XUnmapWindow(xclass.display, button->window);
}




void button_move (Button button, int x, int y)
{
	assert(button);
		
	XMoveWindow(xclass.display, button->window, x, y);
}



void button_reparent (Button button, Window parent, int x, int y)
{
	assert(button);
	if (parent==None) return;

	/* Make sure the button state is correct */
	button->pressd=False;

	XReparentWindow(xclass.display, button->window, parent, x, y);
}




void button_action (Button button, callback action, void *param)
{
	assert(button);

	button->action=action;
	button->param=param;
}

Window button_getwindow (Button button) 
{
	assert(button);

	return button->window;
}
